/*
**  CXSC is a C++ library for eXtended Scientific Computing (V 2.2)
**
**  Copyright (C) 1990-2000 Institut fuer Angewandte Mathematik,
**                          Universitaet Karlsruhe, Germany
**            (C) 2000-2006 Wiss. Rechnen/Softwaretechnologie
**                          Universitaet Wuppertal, Germany   
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Library General Public
**  License as published by the Free Software Foundation; either
**  version 2 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Library General Public License for more details.
**
**  You should have received a copy of the GNU Library General Public
**  License along with this library; if not, write to the Free
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**
**  F. Blomquist, University of Wuppertal, 19.09.2007;
*/
/* CVS $Id: lx_interval.hpp,v 1.2 2006/10/05 10:09:53 cxsc Exp $ */

#ifndef LX_INTERVAL_HP_INCLUDED
#define LX_INTERVAL_HP_INCLUDED

#include <l_interval.hpp>
#include <l_imath.hpp>
#include "lx_real.hpp"

namespace cxsc {

class lx_interval
{
private:
    // ------------- Data Elements -------------------------------------------
    real ex;
    l_interval li;
    // The mathematical value of an object of type lx_interval is
    // interpreted as:  2^(ex) * li;

public:
    // ------------- Constructors --------------------------------------------

    //! Constructor of class lx_interval
    lx_interval(void)  throw() {}

    //! Constructor of class lx_interval
    lx_interval(const real& n, const l_interval& a) throw() 
    {
	if ( !(Is_Integer(n)) ) 
	    cxscthrow(REAL_NOT_ALLOWED("lx_interval(const real&, const l_interval&)"));
	else
	{
	    ex = n; li = a;
	}
    }

    //! Constructor of class lx_interval
    lx_interval(const real& n, const l_real& a) throw() 
    { 
	if ( !(Is_Integer(n)) ) 
	    cxscthrow(REAL_NOT_ALLOWED("lx_interval(const real& n, const l_real& a)"));
	else
	{
	    ex = n; li = a;
	}
    }

    //! Constructor of class lx_interval
    lx_interval(const real& n, const interval& a) throw()
    { 
	if ( !(Is_Integer(n)) ) 
	    cxscthrow(REAL_NOT_ALLOWED("lx_interval(const real&, const interval&)"));
	else
	{
	    ex = n; li = a;
	}
    }

    //! Constructor of class lx_interval
    explicit lx_interval(const real& n, const real& a) throw()
    { 
	if ( !(Is_Integer(n)) ) 
	    cxscthrow(REAL_NOT_ALLOWED("lx_interval(const real&, const real&)"));
	else
	{
	    ex = n; li = a;
	}
    }

    //! Constructor of class lx_interval
    explicit lx_interval(const l_interval& a) throw() : ex(0), li(a) { }
    //! Constructor of class lx_interval
    explicit lx_interval(const l_real& a)     throw() : ex(0), li(a) { }
    //! Constructor of class lx_interval
    lx_interval(const l_real& a, const l_real& b)  throw() : ex(0),li(a,b) { }
    //! Constructor of class lx_interval
    explicit lx_interval(const interval& a)   throw() : ex(0), li(a)   { }
    //! Constructor of class lx_interval
    explicit lx_interval(const real& a)       throw() : ex(0), li(a)   { }
    //! Constructor of class lx_interval
    lx_interval(const lx_real&, const lx_real&) throw();
    //! Constructor of class lx_interval
    explicit lx_interval(const lx_real& a)     
                                 throw() : ex(expo(a)), li(lr_part(a)) { }
    //! Constructor of class lx_interval
    lx_interval(const real&, const string&) throw(); 

    // ------------- Assignments ---------------------------------------------

    //! Implementation of standard assigning operator
    inline lx_interval & operator = (const lx_interval & a) throw();
    //! Implementation of standard assigning operator
    inline lx_interval & operator = (const l_interval & a) throw();
    //! Implementation of standard assigning operator
    inline lx_interval & operator = (const l_real & a)     throw();
    //! Implementation of standard assigning operator
    inline lx_interval & operator = (const real & a)       throw();
    //! Implementation of standard assigning operator
    inline lx_interval & operator = (const interval & a)   throw();
    //! Implementation of standard assigning operator
    inline lx_interval & operator = (const lx_real & a)     throw();

    // l_interval & operator = (const lx_interval &a) throw(); declared
    // in l_interval.hpp and implemented in lx_interval.cpp; 
	 
	 // interval & operator = (const lx_interval &a) throw(); declared
    // in interval.hpp and implemented in lx_interval.cpp; 

    // ------------- Functions -----------------------------------------------

    //! Returns the actual precision of an interval of type lx_interval
    friend inline int StagPrec(const lx_interval&) throw(); 
    //! Returns the exponent of base 2 of an interval of type lx_interval
    friend inline real expo(const lx_interval&) throw();
    //! Returns the l_interval part of an interval of type lx_interval
    friend inline l_interval li_part(const lx_interval&) throw();
    //! The new interval a includs the old one and the old a.li is scaled downwards
    friend void scale_down(lx_interval &a);
    //! The new interval a includs the old one and the old a.li is scaled upwards
    friend void scale_up  (lx_interval &a);

    //! matches the precision of an interval to the actual stagprec value
    friend inline lx_interval adjust(const lx_interval &) throw();
    //! Returns the absolute value of an interval
    friend inline lx_interval abs(const lx_interval &) throw();
    //! Returns 1 if the interval is a point interval
    friend inline bool point_intv(const lx_interval &);
    //! Implementation of standard negation operation
    friend inline bool operator ! (const lx_interval &) throw(); 
    //! Returns 1 if an empty interval is given
    friend inline bool IsEmpty(const lx_interval &) throw(); 
    //! Multiplication of an interval with \f$ 2^n \f$
    friend inline void times2pown(lx_interval &, const real &) throw();
    //! Returns an inflated interval a, if abs(a) is too small
    friend inline lx_interval Blow(const lx_interval &) throw(); 
    //! Multiplication of an interval with \f$ 2^n, n<0, \f$ without integer overflow
    friend inline void times2pown_neg(lx_interval &, const real&) throw();
    //! Returns the relative diameter of an interval
    friend inline lx_real RelDiam( const lx_interval &);
    //! Returns the diameter of an interval
    friend inline lx_real diam(const lx_interval &) throw();
    //! Returns the rounded middle of an interval
    friend inline lx_real mid(const lx_interval &) throw();
    //! Returns the infimum of an interval
    friend inline lx_real Inf(const lx_interval &) throw();
    //! Returns the supremum of an interval
    friend inline lx_real Sup(const lx_interval &) throw();

    // ---------------- Monadic arithmetic operator ---------------------

    //! Implementation of standard algebraic negative sign operation
    friend inline lx_interval operator-(const lx_interval & a) throw();


    // ----------------------- Output -----------------------------------

    //! Implementation of standard output method
    friend std::ostream& operator << (std::ostream&, const lx_interval&) 
	throw();

    //! Implementation of standard output method
    friend std::string & operator << (std::string&, const lx_interval&) 
	throw();

}; // end of class lx_interval

// **********************************************************************
// **********************************************************************

// -------------------------------------------------------------------
//    Declaration of friend functions outside the class lx_interval 
// -------------------------------------------------------------------

inline int StagPrec(const lx_interval &a) throw();
inline real expo(const lx_interval &a) throw();
inline l_interval li_part(const lx_interval &a) throw();

       void scale_down(lx_interval &);
       void scale_up  (lx_interval &);
inline bool point_intv(const lx_interval &);
inline bool IsEmpty(const lx_interval &) throw();
inline void times2pown(lx_interval &, const real &) throw();
inline lx_interval Blow(const lx_interval &) throw();
inline void times2pown_neg(lx_interval &, const real&) throw();
inline lx_real RelDiam(const lx_interval &);
inline lx_real Inf(const lx_interval &) throw();
inline lx_real Sup(const lx_interval &) throw();
inline lx_interval abs(const lx_interval &) throw();
inline lx_interval adjust(const lx_interval &) throw();
inline lx_real diam(const lx_interval &) throw();
inline lx_real mid(const lx_interval &) throw();

// ------------------------ Input --------------------------------------

    //! Implementation of standard input method
    std::string & operator >> (std::string &s, lx_interval &a) throw();
    //! Implementation of standard input method
    void operator >> (const std::string &s, lx_interval &a) throw();
    //! Implementation of standard input method
    void operator >> (const char *s, lx_interval&) throw();

    //! Implementation of standard input method
    std::istream & operator >> (std::istream&, lx_interval&) throw();

// ------------------------ Output --------------------------------------

std::ostream& operator << (std::ostream& s,const lx_interval& a) throw();
// A value a of type lx_interval is written to the 
// output channel as decimal number in the form: 
// { exponent p to base 10, interval mantissa m } = 10^p * m;

std::string & operator << (std::string &s,const lx_interval& a) throw();
// The value of a variable a of type lx_interval is copied to a string s.
// s has the form:  {2**(ex), li} = 2^ex * li;


// -------------------------------------------------------------------
// ------- Function declarations outside the class lx_interval --------
// -------------------------------------------------------------------

void Bin2Dec(const lx_interval& a, real& p, l_interval& m);

inline std::ostream& operator << (std::ostream& s, const lx_real& a) 
    throw(); // A value a of type lx_real is written to the output channel.

//! b = expo2zero(a) returns \f$ a\subseteq b \f$ with \f$ \verb+b.ex+=0 \f$
lx_interval expo2zero(const lx_interval &) throw(OVERFLOW_ERROR);
//! Checks arguments for disjointness
inline int Disjoint(const lx_interval &, const lx_interval &);
//! Checks if first argument lies in the interior of second argument
inline int in (const lx_interval&, const lx_interval&);
//! Checks if first argument lies in the interior of second argument
inline int in (const l_interval&, const lx_interval&);
//! Checks if first argument is part of second argument
inline int in (const interval&, const lx_interval&);

//! Checks if first argument is part of second argument       
inline int in (const lx_real&, const lx_interval&);
//! Checks if first argument is part of second argument       
inline int in (const l_real&, const lx_interval&);
//! Checks if first argument is part of second argument       
inline int in (const real&, const lx_interval&);

//! Returns an epsilon inflation of the first argument
inline lx_interval Blow( const lx_interval&, const real& );
//! Computes the smallest absolute value \f$ \left< \left[x\right] \right> \f$
inline lx_real AbsMin (const lx_interval&); 
//! Computes the greatest absolute value \f$ \left|\left[x\right]\right| \f$  
inline lx_real AbsMax (const lx_interval&); 

// -----------------------------------------------------------------------
// ------------- set comparisons -----------------------------------------
// -----------------------------------------------------------------------

// ---- lx_interval--lx_interval

//! Implementation of standard less-than operation
inline bool operator <  (const lx_interval&, const lx_interval&) throw();
//! Implementation of standard less-or-equal-than operation
inline bool operator <= (const lx_interval&, const lx_interval&) throw();
//! Implementation of standard greater-than operation
inline bool operator >  (const lx_interval&, const lx_interval&) throw();
//! Implementation of standard greater-or-equal-than operation
inline bool operator >= (const lx_interval&, const lx_interval&) throw();

// ---- lx_interval--l_interval

//! Implementation of standard less-than operation
inline bool operator <  (const lx_interval&, const l_interval&) throw();
//! Implementation of standard less-or-equal-than operation
inline bool operator <= (const lx_interval&, const l_interval&) throw();
//! Implementation of standard less-than operation
inline bool operator <  (const l_interval&, const lx_interval&) throw();
//! Implementation of standard less-or-equal-than operation
inline bool operator <= (const l_interval&, const lx_interval&) throw();
//! Implementation of standard greater-than operation
inline bool operator >  (const lx_interval&, const l_interval&) throw();
inline bool operator >= (const lx_interval&, const l_interval&) throw();
//! Implementation of standard greater-than operation
inline bool operator >  (const l_interval&, const lx_interval&) throw();
//! Implementation of standard greater-or-equal-than operation
inline bool operator >= (const l_interval&, const lx_interval&) throw();

// ---- lx_interval--interval

//! Implementation of standard less-than operation
inline bool operator <  (const lx_interval&, const interval&) throw();
//! Implementation of standard less-or-equal-than operation
inline bool operator <= (const lx_interval&, const interval&) throw();
//! Implementation of standard less-than operation
inline bool operator <  (const interval&, const lx_interval&) throw();
//! Implementation of standard less-or-equal-than operation
inline bool operator <= (const interval&, const lx_interval&) throw();
//! Implementation of standard greater-than operation
inline bool operator >  (const lx_interval&, const interval&) throw();
//! Implementation of standard greater-or-equal-than operation
inline bool operator >= (const lx_interval&, const interval&) throw();
//! Implementation of standard greater-than operation
inline bool operator >  (const interval&, const lx_interval&) throw();
//! Implementation of standard greater-or-equal-than operation
inline bool operator >= (const interval&, const lx_interval&) throw();

// ---- lx_interval--real

//! Implementation of standard less-than operation
inline bool operator <  (const real &, const lx_interval &) throw();
//! Implementation of standard less-or-equal-than operation
inline bool operator <= (const real &, const lx_interval &) throw();
//! Implementation of standard greater-than operation
inline bool operator >  (const lx_interval &, const real &) throw();
//! Implementation of standard greater-or-equal-than operation
inline bool operator >= (const lx_interval &, const real &) throw();

// ---- lx_interval--l_real

//! Implementation of standard less-than operation
inline bool operator <  (const l_real &, const lx_interval &) throw();
//! Implementation of standard less-or-equal-than operation
inline bool operator <= (const l_real &, const lx_interval &) throw();
//! Implementation of standard greater-than operation
inline bool operator >  (const lx_interval &, const l_real &) throw();
//! Implementation of standard greater-or-equal-than operation
inline bool operator >= (const lx_interval &, const l_real &) throw();

// ---- lx_interval--lx_real

//! Implementation of standard less-than operation
inline bool operator <  (const lx_real &, const lx_interval &) throw();
//! Implementation of standard less-or-equal-than operation
inline bool operator <= (const lx_real &, const lx_interval &) throw();
//! Implementation of standard greater-than operation
inline bool operator >  (const lx_interval &, const lx_real &) throw();
//! Implementation of standard greater-or-equal-than operation
inline bool operator >= (const lx_interval &, const lx_real &) throw();


// -------------------------- comparisons --------------------------------

//! Implementation of standard negation operation
inline bool operator ! (const lx_interval &) throw();

//! Implementation of standard equality operation
inline bool operator == (const lx_interval &, const lx_interval &) throw();
//! Implementation of standard equality operation
inline bool operator == (const lx_interval &, const l_interval &) throw();
//! Implementation of standard equality operation
inline bool operator == (const l_interval &, const lx_interval &) throw();
//! Implementation of standard equality operation
inline bool operator == (const lx_interval &, const interval &) throw();
//! Implementation of standard equality operation
inline bool operator == (const interval &, const lx_interval &) throw();
//! Implementation of standard equality operation
inline bool operator == (const lx_interval &, const real &) throw();
//! Implementation of standard equality operation
inline bool operator == (const real &, const lx_interval &) throw();
//! Implementation of standard equality operation
inline bool operator == (const lx_interval &, const l_real &) throw();
//! Implementation of standard equality operation
inline bool operator == (const l_real &, const lx_interval &) throw();
//! Implementation of standard equality operation
inline bool operator == (const lx_interval &, const lx_real &) throw();
//! Implementation of standard equality operation
inline bool operator == (const lx_real &, const lx_interval &) throw();

//! Implementation of standard negated equality operation
inline bool operator != (const lx_interval &, const lx_interval &) throw();
//! Implementation of standard negated equality operation
inline bool operator != (const lx_interval &, const l_interval &) throw();
//! Implementation of standard negated equality operation
inline bool operator != (const l_interval &, const lx_interval &) throw();
//! Implementation of standard negated equality operation
inline bool operator != (const lx_interval &, const interval &) throw();
//! Implementation of standard negated equality operation
inline bool operator != (const interval &, const lx_interval &) throw();
//! Implementation of standard negated equality operation
inline bool operator != (const lx_interval &, const real &) throw();
//! Implementation of standard negated equality operation
inline bool operator != (const real &, const lx_interval &) throw();
//! Implementation of standard negated equality operation
inline bool operator != (const lx_interval &, const l_real &) throw();
//! Implementation of standard negated equality operation
inline bool operator != (const l_real &, const lx_interval &) throw();
//! Implementation of standard negated equality operation
inline bool operator != (const lx_interval &, const lx_real &) throw();
//! Implementation of standard negated equality operation
inline bool operator != (const lx_real &, const lx_interval &) throw();

//! Implementation of standard algebraic positive sign operation
inline lx_interval operator+(const lx_interval &) throw();
//! Implementation of standard algebraic negative sign operation
inline lx_interval operator-(const lx_interval &) throw();

//! Implementation of standard algebraic addition operation
lx_interval operator + (const lx_interval &, const lx_interval &) throw();

//! Implementation of standard algebraic addition operation
inline lx_interval operator + (const lx_interval &, const l_interval &) 
                                                                 throw();
//! Implementation of standard algebraic addition operation
inline lx_interval operator + (const l_interval &, const lx_interval &) 
                                                                 throw();
//! Implementation of standard algebraic addition operation
inline lx_interval operator + (const lx_interval &, const l_real &) 
                                                                 throw();
//! Implementation of standard algebraic addition operation
inline lx_interval operator + (const l_real &, const lx_interval &) 
                                                                 throw();
//! Implementation of standard algebraic addition operation
inline lx_interval operator + (const lx_interval &, const lx_real &) 
                                                                 throw();
//! Implementation of standard algebraic addition operation
inline lx_interval operator + (const lx_real &, const lx_interval &) 
                                                                 throw();
//! Implementation of standard algebraic addition operation
inline lx_interval operator + (const lx_interval &, const real &) 
                                                                 throw();
//! Implementation of standard algebraic addition operation
inline lx_interval operator + (const real &, const lx_interval &) 
                                                                 throw();
//! Implementation of standard algebraic addition operation
inline lx_interval operator + (const lx_interval &, const interval &) 
                                                                 throw();
//! Implementation of standard algebraic addition operation
inline lx_interval operator + (const interval &, const lx_interval &)
                                                                 throw();

//! Implementation of standard algebraic addition and allocation operation
inline lx_interval & operator +=(lx_interval &, const lx_interval &) throw();
//! Implementation of standard algebraic addition and allocation operation
inline lx_interval & operator +=(lx_interval &, const l_interval &) throw();
//! Implementation of standard algebraic addition and allocation operation
inline lx_interval & operator +=(lx_interval &, const l_real     &) throw();
//! Implementation of standard algebraic addition and allocation operation
inline lx_interval & operator +=(lx_interval &, const lx_real     &) throw();
//! Implementation of standard algebraic addition and allocation operation
inline lx_interval & operator +=(lx_interval &, const real       &) throw();
//! Implementation of standard algebraic addition and allocation operation
inline lx_interval & operator +=(lx_interval &, const interval   &) throw();

//! Implementation of standard algebraic subtraction operation
inline lx_interval operator - (const lx_interval &, const lx_interval &) 
                                                                 throw();
//! Implementation of standard algebraic subtraction operation
inline lx_interval operator - (const lx_interval &, const l_interval &) 
                                                                 throw();
//! Implementation of standard algebraic subtraction operation
inline lx_interval operator - (const l_interval &, const lx_interval &) 
                                                                 throw();
//! Implementation of standard algebraic subtraction operation
inline lx_interval operator - (const lx_interval &, const l_real &) 
                                                                 throw();
//! Implementation of standard algebraic subtraction operation
inline lx_interval operator - (const l_real &, const lx_interval &) 
                                                                 throw();
//! Implementation of standard algebraic subtraction operation
inline lx_interval operator - (const lx_interval &, const lx_real &) 
                                                                 throw();
//! Implementation of standard algebraic subtraction operation
inline lx_interval operator - (const lx_real &, const lx_interval &) 
                                                                 throw();
//! Implementation of standard algebraic subtraction operation
inline lx_interval operator - (const lx_interval &, const real &) 
                                                                 throw();
//! Implementation of standard algebraic subtraction operation
inline lx_interval operator - (const real &, const lx_interval &) 
                                                                 throw();
//! Implementation of standard algebraic subtraction operation
inline lx_interval operator - (const lx_interval &, const interval &) 
                                                                 throw();
//! Implementation of standard algebraic subtraction operation
inline lx_interval operator - (const interval &, const lx_interval &) 
                                                                 throw();

//! Implementation of standard algebraic subtraction and allocation operation
inline lx_interval & operator -=(lx_interval &, const lx_interval &) throw();
//! Implementation of standard algebraic subtraction and allocation operation
inline lx_interval & operator -=(lx_interval &, const l_interval &) throw();
//! Implementation of standard algebraic subtraction and allocation operation
inline lx_interval & operator -=(lx_interval &, const l_real     &) throw();
//! Implementation of standard algebraic subtraction and allocation operation
inline lx_interval & operator -=(lx_interval &, const lx_real     &) throw();
//! Implementation of standard algebraic subtraction and allocation operation
inline lx_interval & operator -=(lx_interval &, const real       &) throw();
//! Implementation of standard algebraic subtraction and allocation operation
inline lx_interval & operator -=(lx_interval &, const interval   &) throw();

//! Implementation of standard algebraic multiplication operation
lx_interval operator * (const lx_interval &, const lx_interval &) 
    throw();

//! Implementation of standard algebraic multiplication operation
inline lx_interval operator * (const lx_interval &, const l_interval &) 
    throw();
//! Implementation of standard algebraic multiplication operation
inline lx_interval operator * (const l_interval &, const lx_interval &) 
    throw();
//! Implementation of standard algebraic multiplication operation
inline lx_interval operator * (const lx_interval &, const l_real &) 
    throw();
//! Implementation of standard algebraic multiplication operation
inline lx_interval operator * (const l_real &, const lx_interval &) 
    throw();
//! Implementation of standard algebraic multiplication operation
inline lx_interval operator * (const lx_interval &, const lx_real &) 
    throw();
//! Implementation of standard algebraic multiplication operation
inline lx_interval operator * (const lx_real &, const lx_interval &) 
    throw();
//! Implementation of standard algebraic multiplication operation
inline lx_interval operator * (const lx_interval &, const real &) 
    throw();
//! Implementation of standard algebraic multiplication operation
inline lx_interval operator * (const real &, const lx_interval &) 
    throw();
//! Implementation of standard algebraic multiplication operation
inline lx_interval operator * (const lx_interval &, const interval &) 
    throw();
//! Implementation of standard algebraic multiplication operation
inline lx_interval operator * (const interval &, const lx_interval &) 
    throw();

//! Implementation of standard algebraic multiplication and allocation operation
inline lx_interval & operator *=(lx_interval &, const lx_interval &) throw();
//! Implementation of standard algebraic multiplication and allocation operation
inline lx_interval & operator *=(lx_interval &, const l_interval &) throw();
//! Implementation of standard algebraic multiplication and allocation operation
inline lx_interval & operator *=(lx_interval &, const l_real     &) throw();
//! Implementation of standard algebraic multiplication and allocation operation
inline lx_interval & operator *=(lx_interval &, const lx_real     &) throw();
//! Implementation of standard algebraic multiplication and allocation operation
inline lx_interval & operator *=(lx_interval &, const real       &) throw();
//! Implementation of standard algebraic multiplication and allocation operation
inline lx_interval & operator *=(lx_interval &, const interval   &) throw();

//! Implementation of standard algebraic division operation
lx_interval operator / (const lx_interval &, const lx_interval &) 
    throw(ERROR_LINTERVAL_DIV_BY_ZERO);

//! Implementation of standard algebraic division operation
inline lx_interval operator / (const lx_interval &, const l_interval &) 
    throw(ERROR_LINTERVAL_DIV_BY_ZERO);
//! Implementation of standard algebraic division operation
inline lx_interval operator / (const l_interval &, const lx_interval &) 
    throw(ERROR_LINTERVAL_DIV_BY_ZERO);
//! Implementation of standard algebraic division operation
inline lx_interval operator / (const lx_interval &, const l_real &) 
    throw(ERROR_LINTERVAL_DIV_BY_ZERO);
//! Implementation of standard algebraic division operation
inline lx_interval operator / (const l_real &, const lx_interval &) 
    throw(ERROR_LINTERVAL_DIV_BY_ZERO);
//! Implementation of standard algebraic division operation
inline lx_interval operator / (const lx_interval &, const real &) 
    throw(ERROR_LINTERVAL_DIV_BY_ZERO);
//! Implementation of standard algebraic division operation
inline lx_interval operator / (const real &, const lx_interval &) 
    throw(ERROR_LINTERVAL_DIV_BY_ZERO);
//! Implementation of standard algebraic division operation
inline lx_interval operator / (const lx_interval &, const interval &) 
    throw(ERROR_LINTERVAL_DIV_BY_ZERO);
//! Implementation of standard algebraic division operation
inline lx_interval operator / (const interval &, const lx_interval &) 
    throw(ERROR_LINTERVAL_DIV_BY_ZERO);
//! Implementation of standard algebraic division operation
inline lx_interval operator / (const lx_interval &, const lx_real &) 
    throw(ERROR_LINTERVAL_DIV_BY_ZERO);
//! Implementation of standard algebraic division operation
inline lx_interval operator / (const lx_real &, const lx_interval &) 
    throw(ERROR_LINTERVAL_DIV_BY_ZERO);

//! Implementation of standard algebraic division and allocation operation
inline lx_interval & operator /=(lx_interval &, const lx_interval &) throw();
//! Implementation of standard algebraic division and allocation operation
inline lx_interval & operator /=(lx_interval &, const l_interval &) throw();
//! Implementation of standard algebraic division and allocation operation
inline lx_interval & operator /=(lx_interval &, const l_real     &) throw();
//! Implementation of standard algebraic division and allocation operation
inline lx_interval & operator /=(lx_interval &, const lx_real     &) throw();
//! Implementation of standard algebraic division and allocation operation
inline lx_interval & operator /=(lx_interval &, const real       &) throw();
//! Implementation of standard algebraic division and allocation operation
inline lx_interval & operator /=(lx_interval &, const interval   &) throw();

// ----------------------------- Convex hull -------------------------------

    //! Returns the convex hull of the arguments
    inline lx_interval operator | (const lx_interval&, const lx_interval&) 
	throw();
    //! Returns the convex hull of the arguments 
    inline lx_interval operator | (const lx_interval&, const l_interval&) 
	throw();
    //! Returns the convex hull of the arguments 
    inline lx_interval operator | (const l_interval&, const lx_interval&) 
	throw();
    //! Returns the convex hull of the arguments 
    inline lx_interval operator | (const lx_interval&, const interval&) 
	throw();
    //! Returns the convex hull of the arguments
    inline lx_interval operator | (const interval&, const lx_interval&) 
	throw();
    //! Allocates the convex hull of the arguments to the first argument
    inline lx_interval & operator |= (lx_interval&, const lx_interval&) 
	throw();
    //! Allocates the convex hull of the arguments to the first argument
    inline lx_interval & operator |= (lx_interval&, const l_interval&) 
	throw();
    //! Allocates the convex hull of the arguments to the first argument
    inline lx_interval & operator |= (lx_interval&, const interval&) 
	throw();
    //! Returns the convex hull of the arguments
    inline lx_interval operator | (const lx_real&, const lx_interval&) 
	throw();
    //! Returns the convex hull of the arguments
    inline lx_interval operator | (const real&, const lx_interval&) 
	throw();
    //! Returns the convex hull of the arguments
    inline lx_interval operator | (const lx_interval&, const lx_real&) 
	throw();
    //! Returns the convex hull of the arguments
    inline lx_interval operator | (const lx_interval&, const real&) 
	throw();
    //! Returns the convex hull of the arguments
    inline lx_interval operator | (const lx_interval&, const l_real&) 
	throw();
    //! Returns the convex hull of the arguments
    inline lx_interval operator | (const l_real&, const lx_interval&) 
	throw();
    //! Allocates the convex hull of the arguments to the first argument 
    inline lx_interval & operator |= (lx_interval&, const real&)
	throw();
    //! Allocates the convex hull of the arguments to the first argument
    inline lx_interval & operator |= (lx_interval&, const l_real&)
	throw();
    //! Allocates the convex hull of the arguments to the first argument
    inline lx_interval & operator |= (lx_interval&, const lx_real&)
	throw();
    //! Returns the convex hull of the arguments
    inline lx_interval operator | (const lx_real&, const lx_real&) 
	throw();

// --------------------------- Intersection -----------------------------

    //! Returns the intersection of the arguments
    inline lx_interval operator & (const lx_interval&, const lx_interval&) 
	throw(ERROR_LINTERVAL_EMPTY_INTERVAL);
    //! Returns the intersection of the arguments
    inline lx_interval operator & (const lx_interval&, const l_interval&) 
	throw(ERROR_LINTERVAL_EMPTY_INTERVAL); 
    //! Allocates the intersection of the arguments to the first argument
    inline lx_interval & operator &= (lx_interval&, const l_interval&) 
	throw(ERROR_LINTERVAL_EMPTY_INTERVAL); 
    //! Returns the intersection of the arguments
    inline lx_interval operator & (const l_interval&, const lx_interval&) 
	throw(ERROR_LINTERVAL_EMPTY_INTERVAL);
    //! Returns the intersection of the arguments 
    inline lx_interval operator & (const lx_interval&, const interval&) 
	throw(ERROR_LINTERVAL_EMPTY_INTERVAL);
    //! Allocates the intersection of the arguments to the first argument 
    inline lx_interval & operator &= (lx_interval &a, const interval &b) 
	throw(ERROR_LINTERVAL_EMPTY_INTERVAL); 
    //! Returns the intersection of the arguments
    inline lx_interval operator & (const interval&, const lx_interval&) 
	throw(ERROR_LINTERVAL_EMPTY_INTERVAL);
    //! Allocates the intersection of the arguments to the first argument 
    inline lx_interval & operator &= (lx_interval&, const lx_interval&) 
	throw(ERROR_LINTERVAL_EMPTY_INTERVAL); 
    //! Returns the intersection of the arguments
    inline lx_interval operator & (const lx_interval&, const lx_real&) 
	throw(ERROR_LINTERVAL_EMPTY_INTERVAL);
    //! Returns the intersection of the arguments
    inline lx_interval operator & (const lx_interval&, const l_real&) 
	throw(ERROR_LINTERVAL_EMPTY_INTERVAL); 
    //! Returns the intersection of the arguments
    inline lx_interval operator & (const lx_interval&, const real&) 
	throw(ERROR_LINTERVAL_EMPTY_INTERVAL);
    //! Returns the intersection of the arguments 
    inline lx_interval operator & (const lx_real&, const lx_interval&) 
	throw(ERROR_LINTERVAL_EMPTY_INTERVAL);
    //! Returns the intersection of the arguments
    inline lx_interval operator & (const l_real&, const lx_interval&) 
	throw(ERROR_LINTERVAL_EMPTY_INTERVAL); 
    //! Returns the intersection of the arguments
    inline lx_interval operator & (const real&, const lx_interval&) 
	throw(ERROR_LINTERVAL_EMPTY_INTERVAL);
    //! Allocates the intersection of the arguments to the first argument 
    inline lx_interval & operator &= (lx_interval&, const lx_real&) 
	throw(ERROR_LINTERVAL_EMPTY_INTERVAL); 
    //! Allocates the intersection of the arguments to the first argument
    inline lx_interval & operator &= (lx_interval&, const l_real&) 
	throw(ERROR_LINTERVAL_EMPTY_INTERVAL); 
    //! Allocates the intersection of the arguments to the first argument
    inline lx_interval & operator &= (lx_interval&, const real&) 
	throw(ERROR_LINTERVAL_EMPTY_INTERVAL); 

// ------------------------- SetInf, SetSup -----------------------------

//! Returns the interval with the new given infimum value
inline lx_interval & SetInf(lx_interval&, const lx_real&) 
    throw(ERROR_LINTERVAL_EMPTY_INTERVAL);
//! Returns the interval with the new given infimum value
inline lx_interval & SetInf(lx_interval&, const l_real&) 
    throw(ERROR_LINTERVAL_EMPTY_INTERVAL);
//! Returns the interval with the new given infimum value
inline lx_interval & SetInf(lx_interval&, const real&) 
    throw(ERROR_LINTERVAL_EMPTY_INTERVAL);

//! Returns the interval with the new given supremum value
inline lx_interval & SetSup(lx_interval&, const lx_real&) 
    throw(ERROR_LINTERVAL_EMPTY_INTERVAL);
//! Returns the interval with the new given supremum value
inline lx_interval & SetSup(lx_interval&, const l_real&) 
    throw(ERROR_LINTERVAL_EMPTY_INTERVAL);
//! Returns the interval with the new given supremum value
inline lx_interval & SetSup(lx_interval&, const real&) 
    throw(ERROR_LINTERVAL_EMPTY_INTERVAL);

// ------------------------- Help Functions: ----------------------------

// ----------------- Intervals for test calculations -------------------- 

//! Returns a point interval with maximum exponent 1020
l_interval point_max(void);
// returns a staggered point interval with maximum exponent 1020,
// whereby nearly all mantissa bits are set. 

//! Returns a point interval with exponent n
l_interval point_any(int n);
// returns a staggered point interval with exponent n,
// whereby nearly all mantissa bits are set.
// -1074 <= n <= +1020; 

//! Returns a wide interval with maximum exponent 1020
l_interval wide_max(void);
// returns a staggered interval a with maximum exponent 1020
// and diam(a)>0, whereby nearly all mantissa bits are set. 

//! Returns a wide interval with exponent n, \f$ -1074\le n \le +1020; \f$
l_interval wide_any(int n);
// returns a wide staggered interval a with exponent n,
// and diam(a)>0, whereby nearly all mantissa bits are set.
// -1074 <= n <= +1020;


// ------------------- Array of constants ----------------------

// const real ln_N[180];
// ln_N[0] = ln(2); ln_N[1] = ln(3); ... ln_N[179] = ln(181);

const real ln_N[180] =
{6243314768165359.0 / 9007199254740992.0,
   4947709893870347.0 / 4503599627370496.0,
	6243314768165359.0 / 4503599627370496.0,
   7248263982714163.0 / 4503599627370496.0,
	8069367277953026.0 / 4503599627370496.0,
   8763600222181975.0 / 4503599627370496.0,
	4682486076124019.0 / 2251799813685248.0,
   4947709893870347.0 / 2251799813685248.0,
	5184960683398422.0 / 2251799813685248.0,
   5399580128524108.0 / 2251799813685248.0,
   5595512331017853.0 / 2251799813685248.0,
   5775752485243985.0 / 2251799813685248.0,
   5942628803132327.0 / 2251799813685248.0,
   6097986938292255.0 / 2251799813685248.0,
   6243314768165359.0 / 2251799813685248.0,
   6379829280276346.0 / 2251799813685248.0,
   6508538585911686.0 / 2251799813685248.0,
   6630287144694572.0 / 2251799813685248.0,
   6745789375439761.0 / 2251799813685248.0,
   6855655058026161.0 / 2251799813685248.0,
   6960408820565448.0 / 2251799813685248.0,
   7060505291240432.0 / 2251799813685248.0,
   7156341023059193.0 / 2251799813685248.0,
   7248263982714163.0 / 2251799813685248.0,
   7336581177285325.0 / 2251799813685248.0,
   7421564840805520.0 / 2251799813685248.0,
   7503457495173667.0 / 2251799813685248.0,
   7582476122586655.0 / 2251799813685248.0,
   7658815630333595.0 / 2251799813685248.0,
   7732651747257178.0 / 2251799813685248.0,
   7804143460206699.0 / 2251799813685248.0,
   7873435075459281.0 / 2251799813685248.0,
   7940657972317686.0 / 2251799813685248.0,
   8005932102448069.0 / 2251799813685248.0,
   8069367277953026.0 / 2251799813685248.0,
   8131064282924989.0 / 2251799813685248.0,
   8191115836735912.0 / 2251799813685248.0,
   8249607432179158.0 / 2251799813685248.0,
   8306618067481101.0 / 2251799813685248.0,
   8362220887911502.0 / 2251799813685248.0,
   8416483750067501.0 / 2251799813685248.0,
   8469469719751697.0 / 2251799813685248.0,
   8521237512606787.0 / 2251799813685248.0,
   8571841885227428.0 / 2251799813685248.0,
   8621333983281772.0 / 2251799813685248.0,
   8669761652191414.0 / 2251799813685248.0,
   8717169715100533.0 / 2251799813685248.0,
   8763600222181975.0 / 2251799813685248.0,
   8809092674755503.0 / 2251799813685248.0,
   8853684227211519.0 / 2251799813685248.0,
   8897409869326665.0 / 2251799813685248.0,
   8940302591212715.0 / 2251799813685248.0,
   8982393532846860.0 / 2251799813685248.0,
   4511856059940595.0 / 1125899906842624.0,
	4532143093607504.0 / 1125899906842624.0,
   4552071045814873.0 / 1125899906842624.0,
	4571652407313997.0 / 1125899906842624.0,
 4590899028240761.0 / 1125899906842624.0,
 4609822161187467.0 / 1125899906842624.0,
 4628432500714509.0 / 1125899906842624.0,
 4646740219649259.0 / 1125899906842624.0,
 4664755002480667.0 / 1125899906842624.0,
 4682486076124019.0 / 1125899906842624.0,
 4699942238300533.0 / 1125899906842624.0,
 4717131883750310.0 / 1125899906842624.0,
 4734063028474157.0 / 1125899906842624.0,
 4750743332179513.0 / 1125899906842624.0,
 4767180119087803.0 / 1125899906842624.0,
 4783380397244705.0 / 1125899906842624.0,
 4799350876460745.0 / 1125899906842624.0,
 4815097984997183.0 / 1125899906842624.0,
 4830627885101030.0 / 1125899906842624.0,
 4845946487483165.0 / 1125899906842624.0,
 4861059464824668.0 / 1125899906842624.0,
 4875972264388626.0 / 1125899906842624.0,
 4890690119807548.0 / 1125899906842624.0,
 4905218062110249.0 / 1125899906842624.0,
 4919560930046323.0 / 1125899906842624.0,
 4933723379761220.0 / 1125899906842624.0,
 4947709893870347.0 / 1125899906842624.0,
 4961524789976421.0 / 1125899906842624.0,
 4975172228670594.0 / 1125899906842624.0,
 4988656221054420.0 / 1125899906842624.0,
 5001980635816714.0 / 1125899906842624.0,
 5015149205896518.0 / 1125899906842624.0,
 5028165534760914.0 / 1125899906842624.0,
 5041033102324064.0 / 1125899906842624.0,
 5053755270531830.0 / 1125899906842624.0,
 5066335288634384.0 / 1125899906842624.0,
 5078776298167486.0 / 1125899906842624.0,
 5091081337661556.0 / 1125899906842624.0,
 5103253347096176.0 / 1125899906842624.0,
 5115295172116377.0 / 1125899906842624.0,
 5127209568025827.0 / 1125899906842624.0,
 5138999203570936.0 / 1125899906842624.0,
 5150666664528888.0 / 1125899906842624.0,
 5162214457111658.0 / 1125899906842624.0,
 5173645011197227.0 / 1125899906842624.0,
 5184960683398422.0 / 1125899906842624.0,
 5196163759979057.0 / 1125899906842624.0,
 5207256459626429.0 / 1125899906842624.0,
 5218240936088556.0 / 1125899906842624.0,
 5229119280684002.0 / 1125899906842624.0,
 5239893524691621.0 / 1125899906842624.0,
 5250565641627027.0 / 1125899906842624.0,
 5261137549412187.0 / 1125899906842624.0,
 5271611112444100.0 / 1125899906842624.0,
 5281988143568134.0 / 1125899906842624.0,
 5292270405961265.0 / 1125899906842624.0,
 5302459614930081.0 / 1125899906842624.0,
 5312557439628173.0 / 1125899906842624.0,
 5322565504697180.0 / 1125899906842624.0,
 5332485391835543.0 / 1125899906842624.0,
 5342318641298757.0 / 1125899906842624.0,
 5352066753334667.0 / 1125899906842624.0,
 5361731189557166.0 / 1125899906842624.0,
 5371313374261431.0 / 1125899906842624.0,
 5380814695683667.0 / 1125899906842624.0,
 5390236507208137.0 / 1125899906842624.0,
 5399580128524108.0 / 1125899906842624.0,
 5408846846735179.0 / 1125899906842624.0,
 5418037917423337.0 / 1125899906842624.0,
 5427154565669929.0 / 1125899906842624.0,
 5436197987035623.0 / 1125899906842624.0,
 5445169348501337.0 / 1125899906842624.0,
 5454069789371970.0 / 1125899906842624.0,
 5462900422144689.0 / 1125899906842624.0,
 5471662333343435.0 / 1125899906842624.0,
 5480356584321203.0 / 1125899906842624.0,
 5488984212031586.0 / 1125899906842624.0,
 5497546229770980.0 / 1125899906842624.0,
 5506043627892780.0 / 1125899906842624.0,
 5514477374494827.0 / 1125899906842624.0,
 5522848416081301.0 / 1125899906842624.0,
 5531157678200183.0 / 1125899906842624.0,
 5539406066057373.0 / 1125899906842624.0,
 5547594465108473.0 / 1125899906842624.0,
 5555723741629202.0 / 1125899906842624.0,
 5563794743265374.0 / 1125899906842624.0,
 5571808299563294.0 / 1125899906842624.0,
 5579765222481415.0 / 1125899906842624.0,
 5587666306884046.0 / 1125899906842624.0,
 5595512331017853.0 / 1125899906842624.0,
 5603304056971868.0 / 1125899906842624.0,
 5611042231121700.0 / 1125899906842624.0,
 5618727584558574.0 / 1125899906842624.0,
 5626360833503834.0 / 1125899906842624.0,
 5633942679709485.0 / 1125899906842624.0,
 5641473810845338.0 / 1125899906842624.0,
 5648954900873299.0 / 1125899906842624.0,
 5656386610409296.0 / 1125899906842624.0,
 5663769587073346.0 / 1125899906842624.0,
 5671104465828218.0 / 1125899906842624.0,
 5678391869307130.0 / 1125899906842624.0,
 5685632408130919.0 / 1125899906842624.0,
 5692826681215068.0 / 1125899906842624.0,
 5699975276066993.0 / 1125899906842624.0,
 5707078769073944.0 / 1125899906842624.0,
 5714137725781890.0 / 1125899906842624.0,
 5721152701165710.0 / 1125899906842624.0,
 5728124239891016.0 / 1125899906842624.0,
 5735052876567931.0 / 1125899906842624.0,
 5741939135997091.0 / 1125899906842624.0,
 5748783533408181.0 / 1125899906842624.0,
 5755586574691264.0 / 1125899906842624.0,
 5762348756621151.0 / 1125899906842624.0,
 5769070567075090.0 / 1125899906842624.0,
 5775752485243985.0 / 1125899906842624.0,
 5782394981837384.0 / 1125899906842624.0,
 5788998519282460.0 / 1125899906842624.0,
 5795563551917188.0 / 1125899906842624.0,
 5802090526177927.0 / 1125899906842624.0,
 5808579880781584.0 / 1125899906842624.0,
 5815032046902576.0 / 1125899906842624.0,
 5821447448344733.0 / 1125899906842624.0,
 5827826501708347.0 / 1125899906842624.0,
 5834169616552500.0 / 1125899906842624.0,
 5840477195552856.0 / 1125899906842624.0,
 5846749634655054.0 / 1125899906842624.0,
 5852987323223851.0 / 1125899906842624.0};

// --------------------------------------------------------------------
// ----------------- lx_interval Constants ---------------------------- 
 
    //! Enclosure-Interval for \f$ \pi \f$
 lx_interval Pi_lx_interval() throw(); // pi
	 //! Enclosure-Interval for \f$ \pi^2 \f$
 lx_interval Pip2_lx_interval() throw(); // pi^2
	 //! Enclosure-Interval for \f$ 2\pi \f$
 lx_interval Pi2_lx_interval() throw(); // 2*pi
	 //! Enclosure-Interval for \f$ \frac{1}{2\pi} \f$
 lx_interval Pi2r_lx_interval() throw(); // 1/(2*pi)
	 //! Enclosure-Interval for \f$ \pi/4 \f$
 lx_interval Pid4_lx_interval() throw(); // pi/4
	 //! Enclosure-Interval for \f$ \pi/2 \f$
 lx_interval Pid2_lx_interval() throw(); // pi/2
    //! Enclosure-Interval for \f$ \ln(2) \f$
 lx_interval Ln2_lx_interval() throw(); // ln(2)
    //! Enclosure-Interval for \f$ \ln(10) \f$
 lx_interval Ln10_lx_interval() throw(); // ln(10)
    //! Enclosure-Interval for \f$ 1/\ln(10) \f$
 lx_interval Ln10r_lx_interval() throw(); // 1/ln(10)
    //! Enclosure-Interval for \f$ 1/\pi \f$
 lx_interval Pir_lx_interval() throw();  // 1/pi
    //! Enclosure-Interval for \f$ \sqrt{\pi} \f$
 lx_interval SqrtPi_lx_interval() throw();  // sqrt(pi)
    //! Enclosure-Interval for \f$ \sqrt{2\pi} \f$
 lx_interval Sqrt2Pi_lx_interval() throw(); // sqrt(2pi)
    //! Enclosure-Interval for \f$ \sqrt{2} \f$
 lx_interval Sqrt2_lx_interval() throw();  // sqrt(2)
	 //! Enclosure-Interval for \f$ \frac{1}{\sqrt{2}} \f$
 lx_interval Sqrt2r_lx_interval() throw();  // sqrt(2)
    //! Enclosure-Interval for \f$ \sqrt{3} \f$
 lx_interval Sqrt3_lx_interval() throw();  // sqrt(3)
	 //! Enclosure-Interval for \f$ \frac{\sqrt{3}}{2} \f$
 lx_interval Sqrt3d2_lx_interval() throw();  // sqrt(3)/2
	 //! Enclosure-Interval for \f$ \frac{1}{\sqrt{3}} \f$
 lx_interval Sqrt3r_lx_interval() throw();  // 1/sqrt(3)
    //! Enclosure-Interval for \f$ 1/\ln(2) \f$
 lx_interval Ln2r_lx_interval() throw();  // 1/ln(2)
    //! Enclosure-Interval for \f$ \pi/3 \f$
 lx_interval Pid3_lx_interval() throw();  // pi/3
    //! Enclosure-Interval for \f$ 1/\sqrt{\pi} \f$
 lx_interval SqrtPir_lx_interval() throw();  // 1/sqrt(pi)
    //! Enclosure-Interval for \f$ 1/\sqrt{2\pi} \f$
 lx_interval Sqrt2Pir_lx_interval() throw(); // 1/sqrt(2pi)
    //! Enclosure-Interval for \f$ \ln(\pi) \f$
 lx_interval LnPi_lx_interval() throw();  // ln(pi)
    //! Enclosure-Interval for \f$ \ln(2\pi) \f$
 lx_interval Ln2Pi_lx_interval() throw();  // ln(2pi)
    //! Enclosure-Interval for \f$ e=2.718... \f$
 lx_interval E_lx_interval() throw();  // e
	 //! Enclosure-Interval for \f$ e^2 \f$
 lx_interval Ep2_lx_interval() throw();  // e^2
	 //! Enclosure-Interval for \f$ e^{-2} \f$
 lx_interval Ep2r_lx_interval() throw();  // 1/e^2
	 //! Enclosure-Interval for \f$ \frac{1}{e} \f$
 lx_interval Er_lx_interval() throw();  // 1/e
    //! Enclosure-Interval for \f$ e^{\pi} \f$
 lx_interval EpPi_lx_interval() throw();  // e^pi
	 //! Enclosure-Interval for \f$ e^{\pi/2} \f$
 lx_interval EpPid2_lx_interval() throw();  // e^(pi/2)
	 //! Enclosure-Interval for \f$ e^{\pi/4} \f$
 lx_interval EpPid4_lx_interval() throw();  // e^(pi/4)
	 //! Enclosure-Interval for \f$ e^{2\pi} \f$
 lx_interval Ep2Pi_lx_interval() throw();  // e^(2*pi)
    //! Enclosure-Interval for \f$ \mbox{EulerGamma}=0.5772... \f$
 lx_interval EulerGamma_lx_interval() throw();
    //! Enclosure-Interval for \f$ \mbox{Catalan=0.9159... \f$
 lx_interval Catalan_lx_interval() throw();
    //! Enclosure-Interval for \f$ \sqrt{5} \f$ 
 lx_interval sqrt5_lx_interval() throw();  // sqrt(5)
    //! Enclosure-Interval for \f$ \sqrt{7} \f$
 lx_interval sqrt7_lx_interval() throw();   // sqrt(7)
    //! Enclosure-Interval for \f$ 1-2^{-2097} \f$
 lx_interval One_m_lx_interval() throw();
    //! Enclosure-Interval for \f$ 1+2^{-2097} \f$
 lx_interval One_p_lx_interval() throw();

// ---------------- lx_interval Funktionen ---------------------------------

//! Calculates \f$ \sqrt{[x]}  \f$
 lx_interval sqrt(const lx_interval&) throw();
//! Calculates \f$ [x]^2  \f$
 lx_interval sqr(const lx_interval&) throw();
//! Calculates \f$ \ln([x]) \f$
 lx_interval ln(const lx_interval &) throw();
//! Calculates \f$ \log2([x]) \f$
 lx_interval log2(const lx_interval &) throw();
//! Calculates \f$ \log10([x]) \f$
 lx_interval log10(const lx_interval &) throw();
//! Calculates \f$ \ln(1+[x]) \f$
 lx_interval lnp1(const lx_interval &) throw();
//! Calculates \f$ \exp([x]) \f$
 lx_interval exp(const lx_interval &) throw();
//! Calculates \f$ 2^{[x]} \f$
 lx_interval exp2(const lx_interval &) throw(); // 2^x
//! Calculates \f$ 10^{[x]} \f$
 lx_interval exp10(const lx_interval &) throw(); // 10^x
//! Calculates \f$ \exp([x])-1 \f$
 lx_interval expm1(const lx_interval &x) throw(); 
//! Calculates \f$ [x]^n \f$
 lx_interval power(const lx_interval &, const real &) throw();
//! Calculates \f$ [x]^{[y]} \f$
 lx_interval pow(const lx_interval &, const lx_interval &) throw();
//! Calculates \f$ (1+[x])^{[y]} \f$
 lx_interval xp1_pow_y(const lx_interval &, const lx_interval &) throw(); 
//! Calculates \f$ \sin([x]) \f$
 lx_interval sin(const lx_interval &)throw();
//! Calculates \f$ \sin(n\cdot\pi+[x]) \f$
 lx_interval sin_n(const lx_interval &x, const real& n) throw();
//! Calculates \f$ \cos([x]) \f$
 lx_interval cos(const lx_interval &) throw();
//! Calculates \f$ \cos((n+1/2)\cdot\pi+[x]) \f$
 lx_interval cos_n(const lx_interval &x, const real& n) throw();
//! Calculates \f$ \tan([x]) \f$
 lx_interval tan(const lx_interval &) throw();
//! Calculates \f$ \cot([x]) \f$
 lx_interval cot(const lx_interval &) throw();
//! Calculates \f$ \sqrt{1+[x]^2} \f$
 lx_interval sqrt1px2(const lx_interval &) throw();
//! Calculates \f$ \arctan([x]) \f$
 lx_interval atan(const lx_interval &) throw();
//! Calculates \f$ \sqrt{1-[x]^2} \f$
 lx_interval sqrt1mx2(const lx_interval &) throw();
//! Calculates \f$ \sqrt{[x]^2-1} \f$
 lx_interval sqrtx2m1(const lx_interval &) throw();
//! Calculates \f$ \arcsin([x]) \f$
 lx_interval asin(const lx_interval & ) throw();
//! Calculates \f$ \arccos([x]) \f$
 lx_interval acos(const lx_interval &) throw();
//! Calculates \f$ \mbox{arccot}([x]) \f$
 lx_interval acot(const lx_interval &) throw();
//! Calculates \f$ \sinh([x]) \f$
 lx_interval sinh(const lx_interval &) throw();
//! Calculates \f$ \cosh([x]) \f$
 lx_interval cosh(const lx_interval &) throw();
//! Calculates \f$ \tanh([x]) \f$
 lx_interval tanh(const lx_interval &) throw();
//! Calculates \f$ \coth([x]) \f$
 lx_interval coth(const lx_interval &) throw();
//! Calculates \f$ \sqrt{[x]+1}-1} \f$
 lx_interval sqrtp1m1(const lx_interval &) throw();
//! Calculates \f$ \mbox{arcsinh}([x]) \f$
 lx_interval asinh(const lx_interval &) throw();
//! Calculates \f$ \mbox{arccosh}([x]) \f$
 lx_interval acosh(const lx_interval &) throw();
//! Calculates \f$ \mbox{arccosh}(1+[x]) \f$
 lx_interval acoshp1(const lx_interval &) throw();
//! Calculates \f$ \mbox{arctanh}([x]) \f$
 lx_interval atanh(const lx_interval &) throw();
//! Calculates \f$ \mbox{arctanh}(1-[x]) \f$
 lx_interval atanh1m(const lx_interval &) throw();
//! Calculates \f$ \mbox{arctanh}(-1+[x]) \f$
 lx_interval atanhm1p(const lx_interval &) throw();
//! Calculates \f$ \mbox{arccoth}([x]) \f$
 lx_interval acoth(const lx_interval &) throw();
//! Calculates \f$ \mbox{arccoth}(+1+[x]) \f$
 lx_interval acothp1(const lx_interval &) throw();
//! Calculates \f$ \mbox{arctanh}(-1-[x]) \f$
 lx_interval acothm1m(const lx_interval &) throw();
//! Calculates \f$ \sqrt{[x]^2 + [y]^2} \f$
 lx_interval sqrtx2y2(const lx_interval &, const lx_interval &) throw();
//! Calculates \f$ \ln(\sqrt{[x]^2 + [y]^2}) \f$
 lx_interval ln_sqrtx2y2(const lx_interval &, const lx_interval &) throw();
//! Calculates \f$ \sqrt[n]{[x]} \f$
 lx_interval sqrt(const lx_interval &, int) throw();
 
} // end namespace cxsc

#include "lx_interval.inl"

#endif
