//============================================================================
//
//                              Program/Module
//                                   from
//                 C++ TOOLBOX FOR VERIFIED COMPUTING I
//                         Basic Numerical Problems
//
//      Copyright (c) 1995   Rolf Hammer, Matthias Hocks, Dietmar Ratz
//
// This program/module is free software for non-commercial use. For details
// on theory, algorithms, and programs, see the book
//
//  R. Hammer, M. Hocks, U. Kulisch, D. Ratz:  C++ Toolbox for
//  Verified Computing I - Basic Numerical Problems. Springer-Verlag,
//  Heidelberg, New York, 1995.
//
// This program/module is distributed WITHOUT ANY WARRANTY. For details,
// see the "Disclaimer / Legal Matters" of the book (page iv).
//
//============================================================================
//----------------------------------------------------------------------------
// Example: Nonlinear equations in one variable
// This program uses module 'nlfzero' to compute the zeros of the function
//
//              exp(-3*x) - power(sin(x), 3)
//
// A starting interval and a tolerance must be entered.
//----------------------------------------------------------------------------
#include "lx_nlfzero.hpp"  // Nonlinear equations
#include "lxitaylor.hpp"
#include <stacksz.hpp>     // To increase stack size for some
                           // special C++ compilers


using namespace cxsc;
using namespace std;
using namespace taylor;

lx_itaylor f(lx_itaylor x) // Sample function
{
	lx_itaylor res;
//   res = sin(x);
//   res = exp(-3.0*x) - sqr(res)*res;
//     res = exp(-3.0*x) - power(sin(x),3);
//   res = power(x,4) - 12*power(x,3) + 47*power(x,2) -60*x;
//	  res = x*x*x*x - 12*x*x*x + 47*x*x - 60*x;
//   res = x*x*x - 6*sqr(x) +11*x -6;
//   res = sqr(x)*sqr(x-1)*(x-2)*sqr(x-3);
//	  res = sqr(sin(x));
     res = lx_interval(16,l_interval(1))*atan(sqrt(x)) - ln(1+x);
	return res;
}

//----------------------------------------------------------------------------
// Function for prompting and reading information to call the function
// 'AllZeros()'. This function must be called with the function 'f'
// and a string 'Name' containing a textual description of that function.
//----------------------------------------------------------------------------
void compute ( df_FctPtr f )
{
  lx_interval  SearchInterval,hull,fhull,dfhull;
  lx_real      Tolerance;
  lx_ivector   Zero;
  intvector    Unique,UniqueNew;
  int          NumberOfZeros, z, Error, p;
  string s;

  cout << endl << endl << "Computing all zeros of the function implemented in lx_itaylor f(lx_itaylor x) " << endl << endl;
  cout << "stagprec = p defines a precision of 16*p decimal digits." << endl
		 << "p = ? " ;
  cin  >> p;
  if (p<1)  p = 1;
  if (p>30) p = 30;

  stagprec = p;

  cout << "Search interval  {exponent to base 10,[x1,x2]} = ? ";
  cin  >> SearchInterval;
  cout << SetDotPrecision(16,16) << Scientific;
  cout << "SearchInterval = " << SearchInterval << endl;
  cout << endl;
  
  if (p<=8)
	  Tolerance = lx_real(-42*p,l_real(1));
  else 
	  Tolerance = lx_real(-48*p,l_real(1));
  cout << "Tolerance = " << Tolerance << endl << endl;
  cout << endl << "    Please wait some moments:" << endl << endl;
  cout << SetDotPrecision(16*p,16*p) << Scientific;   // Output format  
  AllZeros(f,SearchInterval,Tolerance,Zero,Unique,NumberOfZeros,Error);
  // ***********************************************************************
  // In vector Zero there can be components Zero[m],Zero[n], with
  // m<>n, m,n<=NumberOfZeros fulfilling the conditions:
  // A:
  //    1. The intersection of Zero[m],Zero[n] is not empty,
  //    2. Unique[m] = Unique[n] =1, i.e. Zero[m],Zero[n] include locally
  //                                 unique zeros;
  //    3. f'(x) <> 0 in the convex hull of Zero[m],Zero[n];
  // B:
  //    1. The intersection of Zero[m],Zero[n] is not empty,
  //    2. Unique[m]=1 && Unique[n]=0;
  //    3. Zero[n] <= Zero[m];
  // C: 
  //    1. The intersection of Zero[m],Zero[n] is not empty,
  //    2. Unique[m] = Unique[n]= 0;
  //    3. Zero[n]<=Zero[m]
  // D:
  //    1. The intersection of Zero[m],Zero[n] is not empty,
  //    2. Unique[m] = Unique[n]= 0;
  //    3. Zero[n]<=Zero[m] && Zero[m]<=Zero[n] is not valid
  //
  // In case of A,B,C the component UniqueNew[n] is set to -1.
  // In case of D is realized:
  //    Zero[m] = Zero[m] | Zero[n]; UniqueNew[n] = -1.
  //
  // UniqueNew[j] = -1 signals that a possible zero in Zero[j] is already
  // treated in a component Zero[k], with k<>j and UniqueNew[k] <> -1.
  // Thus, during the output of Zero all componentes Zero[k] with
  // UniqueNew[k] = -1 are overjumped and so (NumberOfZeros - z) is the
  // actual number of detected zeros in the given search interval.
  // ************************************************************************
  z = 0;
  // z counts the number of components in UniqueNew with UniqueNew[j]==-1;
  // The vecors Unique and Zero will not be changed!
  UniqueNew = Unique;

  for (int i=1; i<NumberOfZeros; i++)
  {
	  while (UniqueNew[i]==-1 && i<NumberOfZeros)
		  i = i+1;  // Overjumping treated zeros
	  for (int k=i+1; k<=NumberOfZeros; k++)
	  {
		  while (UniqueNew[k]==-1 && k<NumberOfZeros)
			  k = k+1; // Overjumping treated zeros
		  if (k<=NumberOfZeros)
		  {
			  if ( !(Disjoint(Zero[i],Zero[k])) )
			  {	  
				  if (Unique[i]*Unique[k] == 1)
			  	  {
					  // case A:
				  	  hull = Zero[i] | Zero[k];
				     dfEval(f,hull,fhull,dfhull);
				     if ( !(0.0<=dfhull) )
				     {
					     UniqueNew[k] = -1;
					     z = z+1;
				     }
			  	  }
				  else 
				  {
					  if (Unique[i]==1 && UniqueNew[k]!=-1)
						  if (Zero[k]<=Zero[i])
					     {
							  // case B:
							  UniqueNew[k] = -1;
							  z = z+1;
					     }
					  if (Unique[k]==1 && UniqueNew[i]!=-1)
						  if (Zero[i]<=Zero[k])
					     {
							  // case B:
					 	     UniqueNew[i] = -1;
					 	     z = z+1;
					     } 
						if (Unique[i]==0 && Unique[k]==0)
						{  
							if (Zero[i]<=Zero[k] && UniqueNew[i]!=-1)
							{
								// case C:
								UniqueNew[i] = -1;
								z = z+1;
							}
							else
							  if (Zero[k]<=Zero[i] && UniqueNew[k]!=-1)
							  {
								  // case C:
								  UniqueNew[k] = -1;
								  z = z+1;
							  }
							  else
							  {
								 Zero[i] = Zero[i] | Zero[k];
								 if (UniqueNew[k] != -1)
								 {
									 // case D:
								    UniqueNew[k] = -1; 
									 z = z+1;
								 }
							  }
						 }
				  }
			  }
		  }
	  }
  }
  
  for ( int i=1; i <= NumberOfZeros; i++) 
  {
	  if (UniqueNew[i] != -1)
	  {
		 cout << Zero[i] << endl;
		 if (UniqueNew[i])
			 cout << "encloses a locally unique zero!" << endl;
		 else 
			 cout << "may contain a zero (not verified unique)!" << endl;
	  }
  }
  // NumberOfZeros - z is the number of detected zeros.
  cout << endl << NumberOfZeros - z << " interval enclosure(s)" << endl;

  if (Error) cout << endl << AllZerosErrMsg(Error) << endl;
}

//--------------------
// Body of the program
//--------------------

int main ( )
{
   compute(f);
	return 0;
}
