/*
**  CXSC is a C++ library for eXtended Scientific Computing (V 2.2)
**
**  Copyright (C) 1990-2000 Institut fuer Angewandte Mathematik,
**                          Universitaet Karlsruhe, Germany
**            (C) 2000-2006 Wiss. Rechnen/Softwaretechnologie
**                          Universitaet Wuppertal, Germany   
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Library General Public
**  License as published by the Free Software Foundation; either
**  version 2 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Library General Public License for more details.
**
**  You should have received a copy of the GNU Library General Public
**  License along with this library; if not, write to the Free
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**
**  F. Blomquist, University of Wuppertal, 20.09.2007;
*/
/* CVS $Id: lx_real.hpp,v 1.2 2007/05/01 10:09:53 cxsc Exp $ */

#ifndef LX_REAL_HPP_L_INCLUDED
#define LX_REAL_HPP_L_INCLUDED

#include <iostream>
#include <sstream>  // Wegen stringstream
#include <except.hpp>
#include <l_real.hpp>
#include <cmath>

namespace cxsc {

    const real Max_Int_R =  9007199254740991.0; // = 2^(53) - 1
    const real Max_Int_N = -9007199254738891.0; // = -Max_Int_R+2100
	 
    //! Returns 1 if x is an integer value and if \f$ |x|\le 2^{53}-1 \f$ 
    inline bool Is_Integer(const real& x);
    // returns 1 if x is an integer value and 
    //           if |x| <= 2^53 - 1 = 9007199254740991.0;
    // otherwise 0 is returnd
	 
class lx_real
{

private:
    // ------------- Datenelemente -------------------------------------------
    real ex;
    l_real lr;

public:
    // ------------- Constructors --------------------------------------------

    //! Constructor of class lx_real
    lx_real(void)  throw() {}
    //! Constructor of class lx_real
    lx_real(const real& n, const l_real &a) throw() 
    { 
	if ( !(Is_Integer(n)) ) 
	    cxscthrow(REAL_NOT_ALLOWED("lx_real(const real&, const l_real&)"));
	else
	{
	    ex = n; lr = a;
	}
    }

    //! Constructor of class lx_real
    lx_real(const real& n, const real &a) throw() 
    { 
		if ( !(Is_Integer(n)) ) 
	    	cxscthrow(REAL_NOT_ALLOWED("lx_real(const real&, const real&)"));
		else
		{
	    	ex = n; lr = a;
		}
	 }

    //! Constructor of class lx_real
    explicit lx_real(const l_real &a) throw() : ex(0), lr(a) { }
    //! Constructor of class lx_real
    explicit lx_real(const real &a)   throw() : ex(0), lr(a) { }
	 //! Constructor of class lx_real
	 lx_real(const real&, const string&) throw();


    // ------------- Assignments ---------------------------------------------
	 
    //! Implementation of standard assigning operator
    inline lx_real & operator = (const lx_real &) throw();
	 //! Implementation of standard assigning operator
    inline lx_real & operator = (const l_real &)  throw();
	 //! Implementation of standard assigning operator
    inline lx_real & operator = (const real &)    throw();

    // ------------- Functions -----------------------------------------------

	 //! Returns the precision of a lx_real value
    friend inline int StagPrec(const lx_real&) throw(); 
	 //! Returns the exponent a.ex of a lx_real value a
    friend inline real expo(const lx_real&) throw();
	 //! Returns the sign of a lx_real value
    friend inline int sign(const lx_real&) throw();
	 //! Returns the l_real component a.lr of a lx_real value a
    friend inline l_real lr_part(const lx_real&) throw();
	 //! Returns the absolute value of a lx_real value a
    friend inline lx_real abs(const lx_real&) throw();
	 //! Scaling a.lr upwards and keeps the value of a unchanged
    friend void scale_up  (lx_real &) throw();
	 //! Scaling a.lr downwards and keeps the value of a unchanged
	 friend void scale_down(lx_real &) throw();
	 //! matches the precision of an lx_real value to the actual stagprec value
	 friend inline lx_real adjust(const lx_real &) throw();

	 //! Returns 1 if \f$ a=0 \f$ and 0 otherwise
    friend inline bool eq_zero (const lx_real &a) throw(); // a = 0;
	 //! Returns 1 if \f$ a>0 \f$ and 0 otherwise
    friend inline bool gr_zero (const lx_real &a) throw(); // a > 0;
	 //! Returns 1 if \f$ a\ge0 \f$ and 0 otherwise
    friend inline bool ge_zero (const lx_real &a) throw(); // a >=0;
	 //! Returns 1 if \f$ a<0 \f$ and 0 otherwise
    friend inline bool sm_zero (const lx_real &a) throw(); // a < 0;
	 //! Returns 1 if \f$ a\le0 \f$ and 0 otherwise
    friend inline bool se_zero (const lx_real &a) throw(); // a <=0;
	 
	 //! Multiplication with \f$ 2^r; \f$ the real value r must be an integer
	 friend inline void times2pown(lx_real &, const real &) throw();
	 //! Multiplication with \f$ 2^r; \f$ the real value r must be a negative integer
	 friend inline void times2pown_neg(lx_real& a, const real& n) throw();
	 //! Implementation of standard negation operation
	 friend inline bool operator ! (const lx_real &) throw();
    //! Implementation of standard algebraic negative sign operation
    friend inline lx_real operator -(const lx_real &) throw();


// ----------------------- Output --------------------------------------------

//friend inline std::ostream& operator << (std::ostream& s, const lx_real& a) 
//    throw(); // A value a of type lx_real is written to the output channel.
// The above operator is declared and defined in
// lx_interval.hpp (outside the class lx_interval) , lx_interval.inl;
//! Implementation of standard output method
friend inline std::string & operator << (std::string &s, const lx_real& a) 
    throw();
// The value of a variable a of type lx_real is copied to a string s.
// s has the form:  {2**(ex)*li};  ex is the exponent to base 2.

}; // end of class lx_real

// ------- friend functions declared in class lx_real: ---------

inline int StagPrec(const lx_real&) throw();
inline real expo(const lx_real&) throw();
inline int sign(const lx_real&) throw();
inline l_real lr_part(const lx_real&) throw();
inline lx_real abs(const lx_real&) throw();
inline lx_real adjust(const lx_real &) throw();
inline void times2pown_neg(lx_real&, const real&) throw();

       void scale_up  (lx_real&)   throw();
		 void scale_down(lx_real &a) throw();

inline bool eq_zero (const lx_real &a) throw(); // a = 0;
inline bool gr_zero (const lx_real &a) throw(); // a > 0;
inline bool ge_zero (const lx_real &a) throw(); // a >=0;
inline bool sm_zero (const lx_real &a) throw(); // a < 0;
inline bool se_zero (const lx_real &a) throw(); // a <=0;

inline void times2pown(lx_real &, const real &) throw();
inline bool operator ! (const lx_real &) throw();

inline lx_real operator -(const lx_real &) throw();

// -------------------------- Input ------------------------------------

//! Implementation of standard input method
string & operator >> (string &s, lx_real &a) throw();
//! Implementation of standard input method
void operator >> (const string &s, lx_real &a) throw();
//! Implementation of standard input method
void operator >> (const char *s, lx_real&) throw();
//! Implementation of standard input method
std::istream & operator >> (std::istream &s, lx_real &a) throw();

// -------------------------- Output ------------------------------------

//inline std::ostream& operator << (std::ostream& s, const lx_real& a) 
//    throw(); // A value a of type lx_real is written to the output channel.
//! Implementation of standard output method
inline std::string & operator << (std::string &s, const lx_real& a) 
    throw();
// The value of a variable a of type lx_real is copied to a string s.
// s has the form:  {2**(ex),lr}

// ---- function declarations outside the class lx_real ----

//! Returns \f$ a+b; \f$ a,b must be integers with \f$ |a|,|b|\le2^{53}. \f$
inline real add_real(const real &a, const real &b) throw();

//! Returns \f$ a-b; \f$ a,b must be integers with \f$ |a|,|b|\le2^{53}. \f$
inline real sub_real(const real &a, const real &b) throw();

//! Returns a rather small upper bound of x.
lx_real upper_bnd(const lx_real& x) throw();

//! Returns a rather great lower bound of x.
lx_real lower_bnd(const lx_real& x) throw();
//! Implementation of standard algebraic positive sign operation
inline lx_real operator +(const lx_real &) throw();

//! Implementation of standard algebraic addition operation
lx_real operator + (const lx_real &, const lx_real &) throw();
//! Implementation of standard algebraic addition operation
inline lx_real operator + (const lx_real&, const l_real &) throw();
//! Implementation of standard algebraic addition operation
inline lx_real operator + (const l_real&, const lx_real &) throw();
//! Implementation of standard algebraic addition operation
inline lx_real operator + (const lx_real&, const real &) throw();
//! Implementation of standard algebraic addition operation
inline lx_real operator + (const real&, const lx_real &) throw();

//! Implementation of standard algebraic addition and allocation operation
inline lx_real & operator +=(lx_real &, const lx_real &) throw();
//! Implementation of standard algebraic addition and allocation operation
inline lx_real & operator +=(lx_real &, const l_real &) throw();
//! Implementation of standard algebraic addition and allocation operation
inline lx_real & operator +=(lx_real &, const real &) throw();

//! Implementation of standard algebraic subtraction operation
inline lx_real operator - (const lx_real &, const lx_real &) throw();
//! Implementation of standard algebraic subtraction operation
inline lx_real operator - (const lx_real &, const l_real &) throw();
//! Implementation of standard algebraic subtraction operation
inline lx_real operator - (const l_real &, const lx_real &) throw();
//! Implementation of standard algebraic subtraction operation
inline lx_real operator - (const lx_real &, const real &) throw();
//! Implementation of standard algebraic subtraction operation
inline lx_real operator - (const real &, const lx_real &) throw();

//! Implementation of standard algebraic subtraction and allocation operation
inline lx_real & operator -=(lx_real &, const lx_real &) throw();
//! Implementation of standard algebraic subtraction and allocation operation
inline lx_real & operator -=(lx_real &, const l_real &) throw();
//! Implementation of standard algebraic subtraction and allocation operation
inline lx_real & operator -=(lx_real &, const real &) throw();

//! Implementation of standard algebraic multiplication operation
lx_real operator * (const lx_real &, const lx_real &) throw();
//! Implementation of standard algebraic multiplication operation
inline lx_real operator * (const lx_real&, const l_real &) throw();
//! Implementation of standard algebraic multiplication operation
inline lx_real operator * (const l_real&, const lx_real &) throw();
//! Implementation of standard algebraic multiplication operation
inline lx_real operator * (const lx_real&, const real &) throw();
//! Implementation of standard algebraic multiplication operation
inline lx_real operator * (const real&, const lx_real &) throw();

//! Implementation of standard algebraic multiplication and allocation operation
inline lx_real & operator *=(lx_real &, const lx_real &) throw();
//! Implementation of standard algebraic multiplication and allocation operation
inline lx_real & operator *=(lx_real &, const l_real &) throw();
//! Implementation of standard algebraic multiplication and allocation operation
inline lx_real & operator *=(lx_real &, const real &) throw();

//! Implementation of standard algebraic division operation
lx_real operator / (const lx_real &, const lx_real &) throw(DIV_BY_ZERO);
//! Implementation of standard algebraic division operation
inline lx_real operator / (const lx_real&, const l_real &) throw();
//! Implementation of standard algebraic division operation
inline lx_real operator / (const l_real&, const lx_real &) throw();
//! Implementation of standard algebraic division operation
inline lx_real operator / (const lx_real&, const real &) throw();
//! Implementation of standard algebraic division operation
inline lx_real operator / (const real&, const lx_real &) throw();

//! Implementation of standard algebraic division and allocation operation
inline lx_real & operator /=(lx_real &, const lx_real &) throw();
//! Implementation of standard algebraic division and allocation operation
inline lx_real & operator /=(lx_real &, const l_real &) throw();
//! Implementation of standard algebraic division and allocation operation
inline lx_real & operator /=(lx_real &, const real &) throw();

//! Implementation of standard equality operation
bool operator == (const lx_real &, const lx_real &) throw();
//! Implementation of standard equality operation
inline bool operator == (const lx_real &, const l_real &) throw();
//! Implementation of standard equality operation
inline bool operator == (const l_real &, const lx_real &) throw();
//! Implementation of standard equality operation
inline bool operator == (const lx_real &, const real &)   throw();
//! Implementation of standard equality operation
inline bool operator == (const real &,   const lx_real &) throw();

//! Implementation of standard negated equality operation
inline bool operator != (const lx_real &, const lx_real &) throw();
//! Implementation of standard negated equality operation
inline bool operator != (const lx_real &, const l_real &) throw();
//! Implementation of standard negated equality operation
inline bool operator != (const l_real &, const lx_real &) throw();
//! Implementation of standard negated equality operation
inline bool operator != (const lx_real &, const real &)   throw();
//! Implementation of standard negated equality operation
inline bool operator != (const real &,   const lx_real &) throw();

//! Implementation of standard greater-than operation
bool operator > (const lx_real &, const lx_real &) throw();

//! Implementation of standard less-or-equal-than operation
inline bool operator <= (const lx_real &, const lx_real &) throw();
//! Implementation of standard less-than operation
inline bool operator <  (const lx_real &, const lx_real &) throw();
//! Implementation of standard greater-or-equal-than operation
inline bool operator >= (const lx_real &, const lx_real &) throw();

//! Implementation of standard greater-than operation
inline bool operator >  (const real &, const lx_real &) throw();
//! Implementation of standard less-or-equal-than operation
inline bool operator <= (const real &, const lx_real &) throw();
//! Implementation of standard less-than operation
inline bool operator <  (const real &, const lx_real &) throw();
//! Implementation of standard greater-or-equal-than operation
inline bool operator >= (const real &, const lx_real &) throw();

//! Implementation of standard greater-than operation
inline bool operator >  (const lx_real &, const real &) throw();
//! Implementation of standard less-or-equal-than operation
inline bool operator <= (const lx_real &, const real &) throw();
//! Implementation of standard less-than operation
inline bool operator <  (const lx_real &, const real &) throw();
//! Implementation of standard greater-or-equal-than operation
inline bool operator >= (const lx_real &, const real &) throw();

//! Implementation of standard greater-than operation
inline bool operator >  (const l_real &, const lx_real &) throw();
//! Implementation of standard less-or-equal-than operation
inline bool operator <= (const l_real &, const lx_real &) throw();
//! Implementation of standard less-than operation
inline bool operator <  (const l_real &, const lx_real &) throw();
//! Implementation of standard greater-or-equal-than operation
inline bool operator >= (const l_real &, const lx_real &) throw();

//! Implementation of standard greater-than operation
inline bool operator >  (const lx_real &, const l_real &) throw();
//! Implementation of standard less-or-equal-than operation
inline bool operator <= (const lx_real &, const l_real &) throw();
//! Implementation of standard less-than operation
inline bool operator <  (const lx_real &, const l_real &) throw();
//! Implementation of standard greater-or-equal-than operation
inline bool operator >= (const lx_real &, const l_real &) throw();

//! Calculating the maximum of two lx_real values
inline lx_real max(const lx_real&, const lx_real&);
//! Calculating the minimum of two lx_real values
inline lx_real min(const lx_real&, const lx_real&);

//! Returns the truncated integer part of x.
inline real cutint(const real& x) throw();

// ------------------- lx_real Constants ------------------------------------

    //! lx_real approximation for \f$ \pi \f$
    lx_real Pi_lx_real() throw(); // pi
	 //! lx_real approximation for \f$ \pi^2 \f$
	 lx_real Pip2_lx_real() throw(); // pi^2
	 //! lx_real approximation for \f$ 2\pi \f$
	 lx_real Pi2_lx_real() throw(); // 2*pi
	 //! lx_real approximation for \f$ \frac{\pi}{4} \f$
	 lx_real Pid4_lx_real() throw(); // Pi/4
	 //! lx_real approximation for \f$ \frac{\pi}{2} \f$
	 lx_real Pid2_lx_real() throw(); // Pi/2
    //! lx_real approximation for \f$ \ln(2) \f$
	 lx_real Ln2_lx_real() throw();
    //! lx_real approximation for \f$ \ln(10) \f$
	 lx_real Ln10_lx_real() throw();
	 //! lx_real approximation for \f$ \frac{1}{\ln(10)} \f$
	 lx_real Ln10r_lx_real() throw();
    //! lx_real approximation for \f$ \frac{1}{\pi} \f$
	 lx_real Pir_lx_real() throw();
	 //! lx_real approximation for \f$ \frac{1}{2\pi} \f$
	 lx_real Pi2r_lx_real() throw();  // 1/(2*pi)
    //! lx_real approximation for \f$ \sqrt{\pi} \f$
	 lx_real SqrtPi_lx_real() throw();
    //! lx_real approximation for \f$ \sqrt{2\pi} \f$
	 lx_real Sqrt2Pi_lx_real() throw();
    //! lx_real approximation for \f$ \sqrt{2} \f$
	 lx_real Sqrt2_lx_real() throw();
	 //! lx_real approximation for \f$ \frac{1}{\sqrt{2}} \f$
	 lx_real Sqrt2r_lx_real() throw();
    //! lx_real approximation for \f$ \sqrt{3} \f$
	 lx_real Sqrt3_lx_real() throw();
	 //! lx_real approximation for \f$ \frac{1}{\sqrt{3}} \f$
	 lx_real Sqrt3r_lx_real() throw();
	 //! lx_real approximation for \f$ \frac{\sqrt{3}}{2} \f$
	 lx_real Sqrt3d2_lx_real() throw();
    //! lx_real approximation for \f$ 1/\ln(2) \f$
	 lx_real Ln2r_lx_real() throw();
    //! lx_real approximation for \f$ \pi/3 \f$
	 lx_real Pid3_lx_real() throw();
    //! lx_real approximation for \f$ 1/\sqrt{\pi} \f$
	 lx_real SqrtPir_lx_real() throw();
    //! lx_real approximation for \f$ 1/\sqrt{2\pi} \f$
	 lx_real Sqrt2Pir_lx_real() throw();
    //! lx_real approximation for \f$ \ln(\pi) \f$
	 lx_real LnPi_lx_real() throw();
    //! lx_real approximation for \f$ \ln(2\pi) \f$
	 lx_real Ln2Pi_lx_real() throw();
    //! lx_real approximation for \f$ e=2.718... \f$
	 lx_real E_lx_real() throw();
	 //! lx_real approximation for \f$ e^2 \f$
	 lx_real Ep2_lx_real() throw();
	 //! lx_real approximation for \f$ \frac{1}{e^2} \f$
	 lx_real Ep2r_lx_real() throw();
	 //! lx_real approximation for \f$ \frac{1}{e} \f$
	 lx_real Er_lx_real() throw();
    //! lx_real approximation for \f$ e^{\pi} \f$
	 lx_real EpPi_lx_real() throw();
	 //! lx_real approximation for \f$ e^{\pi/2} \f$
	 lx_real EpPid2_lx_real() throw();
	 //! lx_real approximation for \f$ e^{\pi/4} \f$
	 lx_real EpPid4_lx_real() throw();
	 //! lx_real approximation for \f$ e^{2\pi} \f$
	 lx_real Ep2Pi_lx_real() throw();
    //! lx_real approximation for \f$ \mbox{EulerGamma}=0.5772... \f$
	 lx_real EulerGamma_lx_real() throw();
    //! lx_real approximation for \f$ \mbox{Catalan=0.9159... \f$
	 lx_real Catalan_lx_real() throw();
    //! lx_real approximation for \f$ \sqrt{5} \f$
	 lx_real sqrt5_lx_real() throw();
    //! lx_real approximation for \f$ \sqrt{7} \f$
	 lx_real sqrt7_lx_real() throw();
    //! lx_real approximation for \f$ 1-2^{-2097} \f$
	 lx_real One_m_lx_real() throw();
    //! lx_real approximation for \f$ 1+2^{-2097} \f$
	 lx_real One_p_lx_real() throw();


}  // end namespace cxsc

#include "lx_real.inl"

#endif
